"""
Code relating to the command line interface to acclimatise
"""
import argparse
import subprocess
import sys
import typing

from acclimatise.converter import WrapperGenerator, cases
from acclimatise.model import Command
from acclimatise.parser import best_cmd, parse_help
from pyparsing import ParseBaseException


def main():
    args = get_parser().parse_args()

    # Allow input of help text either by running the command, or using stdin
    if args.stdin:
        stdin = "".join(sys.stdin.readlines())
        command = parse_help(args.cmd, stdin)
    else:
        command = best_cmd(args.cmd)

    converter_cls = WrapperGenerator.choose_converter(args.format)
    converter = converter_cls(
        generate_names=args.generate_names,
        ignore_positionals=args.no_pos,
        case=args.case,
    )
    output = converter.generate_wrapper(command)
    print(output)


def get_parser() -> argparse.ArgumentParser:
    parser = argparse.ArgumentParser()
    parser.add_argument("cmd", nargs="+", help="The base command this tool uses")
    parser.add_argument(
        "-f",
        "--format",
        choices=["wdl", "cwl"],
        required=True,
        help="The language in which to output the CLI wrapper",
    )
    parser.add_argument(
        "--no-pos",
        action="store_true",
        help=(
            "Don't include positional arguments, for example because the help formatting has some "
            "misleading sections that look like positional arguments"
        ),
    )
    parser.add_argument(
        "-g",
        "--generate-names",
        action="store_true",
        help=(
            "Rather than using the long flag to generate the argument name, generate them automatically using the "
            "flag description. Generally helpful if there are no long flags, only short flags."
        ),
    )
    parser.add_argument(
        "-c",
        "--case",
        choices=cases,
        help=(
            "Which case to use for variable names. If not set, defaults to the language defaults: snake_case for CWL"
            " and snake_case for WDL"
        ),
        default="snake",
    )
    parser.add_argument(
        "--stdin",
        help="Accept the help text from stdin instead of running the command",
        action="store_true",
    )
    return parser


if __name__ == "__main__":
    main()



import subprocess
import typing

from acclimatise.flag_parser.parser import CliParser
from acclimatise.model import Command
from acclimatise.usage_parser import parse_usage
from pyparsing import ParseBaseException


def parse_help(cmd: typing.Collection[str], text: str, parse_positionals=True):
    help_command = CliParser(parse_positionals=parse_positionals).parse_command(
        name=cmd, cmd=text
    )
    usage_command = parse_usage(cmd, text)

    # Normally parsing the list of flags will provide a better command summary, but if it didn't give us anything,
    # fall back to the usage
    if len(help_command.positional) + len(help_command.named) == 0:
        command = usage_command
    else:
        command = help_command

        # However, even if we aren't referring to the usage for the whole command, we can cross-check to ensure we're
        # using some correct arguments
        if len(usage_command.positional) != len(help_command.positional):
            command.positional = usage_command.positional
        # confirmed = {pos.name for pos in usage_command.positional}
        # for positional in command.positional:
        #     if positional.name in confirmed:
        #         positional.usage_supported = True

        # Then, by default we filter out unsupported positionals, to remove false positives
        # if only_supported_positionals:
        #     command.positional = [pos for pos in command.positional if pos.usage_supported]

    return command


def best_cmd(
    cmd: typing.List[str],
    flags: typing.Iterable[str] = ([], ["-h"], ["--help"], ["--usage"]),
) -> Command:
    """
    Determine the best Command instance for a given command line tool, by trying many
    different help flags, such as --help and -h
    :param cmd: The command to analyse, e.g. ['wc'] or ['bwa', 'mem']
    :param flags: A list of help flags to try, e.g. ['--help', '-h']
    """
    # For each help flag, run the command and then try to parse it
    commands = []
    for flag in flags:
        help_cmd = cmd + flag
        final = execute_cmd(help_cmd)
        try:
            commands.append(parse_help(cmd, final))
        except ParseBaseException:
            # If parsing fails, this wasn't the right flag to use
            continue

    return max(commands, key=lambda com: len(com.named) + len(com.positional))


def execute_cmd(help_cmd: typing.List[str]) -> str:
    """
    Execute a command defined by a list of arguments, and return the result as a string
    """
    try:
        proc = subprocess.run(
            help_cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE, timeout=1
        )
        return (proc.stdout or proc.stderr).decode("utf_8")
    except subprocess.TimeoutExpired:
        return ""



"""
Utilities for producing workflow wrappers using jinja templates
"""
import jinja2
from acclimatise import model


def get_env() -> jinja2.Environment:
    """
    Returns a Jinja2 environment with some helpful filters
    :return:
    """
    env = jinja2.Environment(
        loader=jinja2.PackageLoader("acclimatise", "converter"),
        autoescape=jinja2.select_autoescape(["html", "xml"]),
    )

    return env



from acclimatise.parser import parse_help


def test_pisces(pisces_help):
    command = parse_help(cmd=["pisces"], text=pisces_help)
    assert len(command.named) == 57


def test_htseq(htseq_help):
    command = parse_help(cmd=["htseq-count"], text=htseq_help)
    assert len(command.named) == 14
    assert len(command.positional) == 2


def test_bwa(bwamem_help):
    command = parse_help(cmd=["bwa", "mem"], text=bwamem_help)
    assert len(command.named) == 32
    assert len(command.positional) == 3
